<?php

namespace Drupal\cms_content_sync\SyncCoreInterface;

use Drupal\cms_content_sync\Entity\Pool;
use EdgeBox\SyncCore\V1\SyncCore;

/**
 * Class SyncCoreFactory.
 *
 * This is the central interface between Drupal and the Sync Core. Drupal will call ::getSyncCore() and then use the
 * Sync Core library whereas the Sync Core library in return uses the interface functions provided here.
 */
class SyncCoreFactory {

  /**
   * @var \EdgeBox\SyncCore\Interfaces\ISyncCore[]
   */
  protected static $clients = [];

  /**
   * Return an instance of the ISyncCore Client for the given URL.
   * Cache clients.
   *
   * @param string $sync_core_url
   *   The base URL to the Sync Core.
   *
   * @return \EdgeBox\SyncCore\Interfaces\ISyncCore
   */
  public static function getSyncCore($sync_core_url) {
    if (!empty(self::$clients[$sync_core_url])) {
      return self::$clients[$sync_core_url];
    }

    return self::$clients[$sync_core_url] = new SyncCore(DrupalApplication::get(), $sync_core_url);
  }

  /**
   * @var \EdgeBox\SyncCore\Interfaces\ISyncCore[]
   */
  protected static $allSyncCores = NULL;

  /**
   * @return \EdgeBox\SyncCore\Interfaces\ISyncCore[]
   */
  public static function getAllSyncCores() {
    if (!empty(self::$allSyncCores)) {
      return self::$allSyncCores;
    }

    $cores = [];

    foreach (Pool::getAll() as $pool) {
      $url = parse_url($pool->getSyncCoreUrl());
      $host = $url['host'];
      if (isset($cores[$host])) {
        continue;
      }

      $cores[$host] = self::getSyncCore($pool->getSyncCoreUrl());
    }

    return self::$allSyncCores = $cores;
  }

  /**
   * @return \EdgeBox\SyncCore\Interfaces\ISyncCore|null
   */
  public static function getAnySyncCore() {
    $cores = self::getAllSyncCores();
    if (!count($cores)) {
      return NULL;
    }
    return reset($cores);
  }

}
